import LeaderModelManager from '/base-standard/ui/diplomacy/leader-model-manager.js';

const LEADER_FALLBACK_GAME_ASSET = 'LEADER_FALLBACK_GAME_ASSET';

function getRandomInt(min, max) {
    return Math.floor(Math.random() * (max - min + 1)) + min;
}

LeaderModelManager.findModelConfig = function (_model){
    if (!("modelMap" in this)) {
        return null;
    }

    const model = _model.replace('_GAME_ASSET', '');

    if (model in this.modelMap) {
        return this.modelMap[model];
    }

    for (const config of Object.values(LeaderModelManager.modelMap)) {
        if (config === model) {
            return config;
        }

        if(typeof config === 'object' && 'model' in config && config.model === model) {
            return config;
        }
    }

    return null;
}

LeaderModelManager.hasModelConfig = function (model) {
    return !!this.findModelConfig(model);
}

LeaderModelManager.getModelConfig = function (model, side = 'left') {
    let config = this.findModelConfig(model);

    if (typeof config === 'string') {
        config = { model: config };
    }

    console.error(model, config)

    let angle = config.angle || [0, 0];
    if (!Array.isArray(angle)) {
        angle = [angle, -1 * angle];
    }
    if (side === 'right') {
        angle = angle[1];
    } else {
        angle = angle[0];
    }

    return {
        model: config.model,
        animation: config?.animation || (!config.model.includes('LEAD') ? 'IDLE' : undefined),
        params: {
            ...(config.seed !== undefined ? { seed: config.seed } : { seed: getRandomInt(-2147483646, 2147483646) }),
            ...(config.scale !== undefined ? { scale: config.scale } : {}),
            ...(config.alpha !== undefined ? { alpha: config.alpha } : {}),
            angle,
            selectionScriptParams: {
                ...(config.age !== undefined ? { age: config.age } : {}),
                ...(config.civilization !== undefined ? { civilization: config.civilization } : {})
            }
        },
    }
}

LeaderModelManager.getLeaderAssetNameBase = LeaderModelManager.getLeaderAssetName;
LeaderModelManager.getLeaderAssetName = function (model) {
    const currentTimestamp = Date.now();
    if(timestamp + 100 < currentTimestamp) {
        timestamp = currentTimestamp;
        assetNames = [];
    }
    if (this.hasModelConfig(model)) {
        assetNames.push(model);
        return LEADER_FALLBACK_GAME_ASSET;
    }

    return LeaderModelManager.getLeaderAssetNameBase(model);
}

let timestamp = Date.now();
let assetNames = [];
LeaderModelManager.playLeaderAnimation = function (_stateName, leaderSide) {
    let stateName = _stateName;
    if (leaderSide.toLowerCase() == "left") {
        if (this.leader3DModelLeft == null) {
            return;
        }

        console.error(`LEADER: ${this.leader3DModelLeft.assetName}`)
        if (this.leader3DModelLeft.assetName === LEADER_FALLBACK_GAME_ASSET &&assetNames.length > 0 && this.hasModelConfig(assetNames[0])) {
            const config = this.getModelConfig(assetNames[0], leaderSide.toLowerCase());

            console.error(`SEED: ${config?.params?.seed}`)

            this.leader3DModelLeft.setAssetNameAndParams(config.model, config.params || {});
            if (config?.params?.selectionScriptParams) {
                this.leader3DModelLeft.setSelectionScriptParams(config.params.selectionScriptParams);
                this.leader3DModelLeft.updateSelectionScriptParams(config.params.selectionScriptParams);
            }

            stateName = config?.animation || stateName;
        }

        this.leader3DModelLeft.setState(stateName);
        this.leaderLeftAnimationJustStarted = true;
        this.leftAnimState = stateName;
        this.leftAnimationStartTime = performance.now();

    } else if (leaderSide.toLowerCase() == "right") {
        if (this.leader3DModelRight == null) {
            return;
        }

        if (this.leader3DModelRight.assetName === LEADER_FALLBACK_GAME_ASSET && assetNames.length > 0 && this.hasModelConfig(assetNames[1] || assetNames[0])) {
            const config = this.getModelConfig(assetNames[1] || assetNames[0], leaderSide.toLowerCase());

            console.error(`SEED: ${config?.params?.seed}`)

            this.leader3DModelRight.setAssetNameAndParams(config.model, config.params || {});
            if (config?.params?.selectionScriptParams) {
                this.leader3DModelRight.setSelectionScriptParams(config.params.selectionScriptParams);
                this.leader3DModelRight.updateSelectionScriptParams(config.params.selectionScriptParams);
            }

            stateName = config?.animation || stateName;
        }

        this.leader3DModelRight.setState(stateName);
        this.leaderRightAnimationJustStarted = true;
        this.rightAnimState = stateName;
        this.rightAnimationStartTime = performance.now();
    }
}

LeaderModelManager.showLeftLeaderModelBase = LeaderModelManager.showLeftLeaderModel;
LeaderModelManager.showLeftLeaderModel = function (playerID) {
    this.showLeftLeaderModelBase(playerID);
    this.playLeaderAnimation("IDLE", "left");
}
