import LeaderSelectModelManager from '/core/ui/shell/leader-select/leader-select-model-manager.js';

function getRandomInt(min, max) {
    return Math.floor(Math.random() * (max - min + 1)) + min;
}

LeaderSelectModelManager.hasModelConfig = function (model) {
    if (!("modelMap" in this)) {
        return false;
    }

    if (model in this.modelMap) {
        return true;
    }

    if (Object.values(this.modelMap).some(config => config === model || config.model === model)) {
        return true;
    }

    return false;
}

LeaderSelectModelManager.getModelConfig = function (model) {
    if (this.hasModelConfig(model)) {
        let config = model in this.modelMap
            ? this.modelMap[model]
            : Object.values(this.modelMap).find(config => config === model || config.model === model);

        if (typeof config === 'string') {
            config = { model: config };
        }

        return {
            model: config.model,
            animation: config?.animation || (!config.model.includes('LEAD') ? 'IDLE' : undefined),
            params: {
                ...(config.seed !== undefined ? { seed: config.seed } : { seed: getRandomInt(-2147483646, 2147483646) }),
                ...(config.scale !== undefined ? { scale: config.scale } : {}),
                ...(config.alpha !== undefined ? { alpha: config.alpha } : {}),
                ...(config.angle !== undefined ? { angle: config.angle } : {}),
            },
        }
    }

    return { model, params: {} };
}

LeaderSelectModelManager.getLeaderAssetNameBase = LeaderSelectModelManager.getLeaderAssetName;
LeaderSelectModelManager.getLeaderAssetName = function (model) {
    replaceAssetName = 'LEADER_FALLBACK_GAME_ASSET';

    if (model.indexOf('__') > -1) {
        replaceAssetName = model.split('__')[1];
    } else if (this.hasModelConfig(model)) {
        replaceAssetName = model;
    }

    return LeaderSelectModelManager.getLeaderAssetNameBase(model);
}

let replaceAssetName = 'LEADER_FALLBACK_GAME_ASSET';
LeaderSelectModelManager.playLeaderAnimation = function (_stateName) {
    let stateName = _stateName;

    if (this.leader3DModel == null) {
        return;
    }

    if (this.hasModelConfig(replaceAssetName)) {
        const config = this.getModelConfig(replaceAssetName);

        console.error(`SEED: ${config?.params?.seed}`);

        this.leader3DModel.setAssetNameAndParams(config.model, config.params || {});

        stateName = config?.animation || stateName;
    }

    this.leader3DModel.setState(stateName);
    this.currentLeaderAnimationState = stateName;
    this.leaderAnimationJustStarted = true;
}
